@@ -1,19 +1,3 @@
-#ifndef CONVERT_PATH
-#define CONVERT_PATH "/usr/local/bin"
-#endif
-
-#ifndef NETPBM_PATH
-#define NETPBM_PATH  "/usr/local/bin"
-#endif
-
-#ifndef CJPEG_PROG
-#define CJPEG_PROG "/usr/local/bin/cjpeg"
-#endif
-
-#ifndef DJPEG_PROG
-#define DJPEG_PROG "/usr/local/bin/djpeg"
-#endif
-
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -141,9 +125,6 @@ void                calc_map_tables(ImlibData * id, ImlibImage * im);
 
 void                _PaletteAlloc(ImlibData * id, int num, int *cols);
 
-FILE               *open_helper(const char *, const char *, const char *);
-int                 close_helper(FILE *);
-
 #define INDEX_RGB(r,g,b)  id->fast_rgb[(r<<10)|(g<<5)|(b)]
 #define COLOR_INDEX(i)    id->palette[i].pixel
 #define COLOR_RGB(r,g,b)  id->palette[INDEX_RGB(r,g,b)].pixel
