@@ -39,6 +39,17 @@ _GetExtension(char *file)
     return p + 1;
 }
 
+/*
+ *     Make sure we don't wrap on our memory allocations
+ */
+
+void * _imlib_malloc_image(unsigned int w, unsigned int h)
+{
+       if( w > 32767 || h > 32767)
+               return NULL;
+       return malloc(w * h * 3);
+}
+
 #ifdef HAVE_LIBJPEG
 
 /** 
@@ -96,7 +107,7 @@ _LoadJPEG(ImlibData * id, FILE * f, int *w, int *h)
   jpeg_start_decompress(&cinfo);
   *w = cinfo.output_width;
   *h = cinfo.output_height;
-  data = malloc(*w ** h * 3);
+  data = _imlib_malloc_image(*w, *h);
   if (!data)
     {
       jpeg_destroy_decompress(&cinfo);
@@ -208,12 +219,13 @@ _LoadPNG(ImlibData * id, FILE * f, int *w, int *h, int *t)
   if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
     png_set_expand(png_ptr);
   png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);
-  data = malloc(*w ** h * 3);
+  data = _imlib_malloc_image(*w, *h);
   if (!data)
     {
       png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
       return NULL;
     }
+    /* This malloc is fine, _imlib_malloc_image will fail first */
   lines = (unsigned char **)malloc(*h * sizeof(unsigned char *));
 
   if (lines == NULL)
@@ -225,6 +237,7 @@ _LoadPNG(ImlibData * id, FILE * f, int *w, int *h, int *t)
 
   for (i = 0; i < *h; i++)
     {
+     /* Safe as well for the same reason as above */
       if ((lines[i] = malloc(*w * (sizeof(unsigned char) * 4))) == NULL)
 	{
 	  int                 n;
@@ -347,6 +360,11 @@ _LoadTIFF(ImlibData * id, FILE *f, char *file, int *w, int *h, int *t)
   npix = ww * hh;
   *w = (int)ww;
   *h = (int)hh;
+  if(ww > 32767 || hh > 32767)
+    {
+       TIFFClose(tif);
+       return NULL;
+    }
   rast = (uint32 *) _TIFFmalloc(npix * sizeof(uint32));
   if (!rast)
     {
@@ -445,13 +463,17 @@ _LoadGIF(ImlibData * id, FILE *f, int *w, int *h, int *t)
 	    }
 	  *w = gif->Image.Width;
 	  *h = gif->Image.Height;
+	  if (*h > 32767 || *w > 32767)
+	    {
+	       return NULL;
+	    }
 	  rows = malloc(*h * sizeof(GifRowType *));
 	  if (!rows)
 	    {
 	      DGifCloseFile(gif);
 	      return NULL;
 	    }
-	  data = malloc(*w ** h * 3);
+	  data = _imlib_malloc_image(*w, *h);
 	  if (!data)
 	    {
 	      DGifCloseFile(gif);
@@ -657,7 +679,7 @@ _LoadBMP(ImlibData * id, FILE *file, int *w, int *h, int *t)
     }
   else
     cmap = NULL;
-  ptr = (unsigned char *)malloc(*w * *h * 3);
+  ptr = (unsigned char *)_imlib_malloc_image(*w, *h);
   if (!ptr)
     {
       fprintf(stderr, "IMLIB ERROR: Cannot allocate RAM for RGB data in file\n");
@@ -1003,6 +1025,7 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 		      free(line);
 		      return NULL;
 		    }
+		    /* SAFE -- Check for w,h in earlier code */
 		  data = malloc(*w ** h * 3);
 		  if (!data)
 		    {
@@ -1038,7 +1061,7 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 			  if (line[k] != ' ')
 			    {
 			      s[0] = 0;
-			      sscanf(&line[k], "%65535s", s);
+			      sscanf(&line[k], "%256s", s);
 			      slen = strlen(s);
 			      k += slen;
 			      if (!strcmp(s, "c"))
@@ -1252,6 +1275,11 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 	{
 	  lsz += 256;
 	  line = realloc(line, lsz);
+	  if(line == NULL)
+	    {
+	    	free(cmap);
+	    	return NULL;
+	    }
 	}
       if ((ptr) && ((ptr - data) >= *w ** h * 3))
 	done = 1;
@@ -1330,7 +1358,7 @@ _LoadPPM(ImlibData * id, FILE * f, int *w, int *h)
 	    }
 	  else
 	    scale = 99999;
-	  ptr = (unsigned char *)malloc(a * b * 3);
+	  ptr = (unsigned char *)_imlib_malloc_image(a, b);
 	  if (!ptr)
 	    {
 	      fprintf(stderr, "IMLIB ERROR: Cannot allocate RAM for RGB data in file");
@@ -1517,17 +1545,17 @@ ispnm(FILE *f)
     return 0;
   fgets(buf, 8, f);
   rewind(f);
-  if (!strcmp("P6\n", buf))
+  if (!strcmp("P6", buf))
     return 1;
-  if (!strcmp("P5\n", buf))
+  if (!strcmp("P5", buf))
     return 1;
-  if (!strcmp("P4\n", buf))
+  if (!strcmp("P4", buf))
     return 1;
-  if (!strcmp("P3\n", buf))
+  if (!strcmp("P3", buf))
     return 1;
-  if (!strcmp("P2\n", buf))
+  if (!strcmp("P2", buf))
     return 1;
-  if (!strcmp("P1\n", buf))
+  if (!strcmp("P1", buf))
     return 1;
   return 0;
 }
@@ -1642,7 +1670,6 @@ ImlibImage         *
 Imlib_load_image(ImlibData * id, char *file)
 {
   int                 w, h;
-  int                 needs_conv = 1;
   unsigned char      *data;
   ImlibImage         *im;
   char                s[4096];
@@ -1686,71 +1713,38 @@ Imlib_load_image(ImlibData * id, char *file)
 
   if (ispnm(p))
     {
-      needs_conv = 0;
       fmt = 0;
     }
   else if (isjpeg(p))
     {
-#ifdef HAVE_LIBJPEG
-      needs_conv = 0;
       fmt = 2;
-#else
-      needs_conv = 1;
-      fmt = 0;
-#endif
     }
   else if (istiff(p))
     {
-#ifdef HAVE_LIBTIFF
-      needs_conv = 0;
       fmt = 3;
-#else
-      needs_conv = 1;
-      fmt = 0;
-#endif
     }
   else if (iseim(p))
     {
-      needs_conv = 0;
       eim = 1;
       fmt = 9999;
     }
   else if (isxpm(p))
     {
-      needs_conv = 0;
       fmt = 5;
     }
   else if (ispng(p))
     {
-#ifdef HAVE_LIBPNG
-      needs_conv = 0;
       fmt = 1;
-#else
-      needs_conv = 1;
-      fmt = 0;
-#endif
     }
   else if (isgif(p))
     {
-#ifdef HAVE_LIBGIF
-      needs_conv = 0;
       fmt = 4;
-#else
-      needs_conv = 1;
-      fmt = 0;
-#endif
     }
   else if (isbmp(p))
     {
-      needs_conv = 0;
       fmt = 6;
     }
-  if ((needs_conv) && (id->fallback))
-    {
-      if (p != stdin) 
-        fclose(p);
-      p = open_helper("%C/convert %s pnm:-", fil, "rb");
-    }
+
   trans = 0;
   if ((!eim) && (!data))
     {
@@ -1787,102 +1781,14 @@ Imlib_load_image(ImlibData * id, char *file)
 	  break;
 	}
     }
-  if ((p) && (!needs_conv))
-    {
-      if (p != stdin) 
-        fclose(p);
-    }
-  else if (p)
-    close_helper(p);
 
-  if ((!data) && (id->fallback))
-    {
-      p = open_helper("%C/convert %s pnm:-", fil, "rb");
-      if (p)
-	{
-	  data = _LoadPPM(id, p, &w, &h);
-	  close_helper(p);
-	}
-    }
-  if ((!eim) && (!data) && (id->fallback))
-    {
-      if (!strcasecmp(s, "jpeg"))
-	strcpy(cmd, "%J %s");
-      else if (!strcasecmp(s, "jpg"))
-	strcpy(cmd, "%J %s");
-      else if (!strcasecmp(s, "bmp"))
-	strcpy(cmd, "%P/bmptoppm %s");
-      else if (!strcasecmp(s, "ilbm"))
-	strcpy(cmd, "%P/ilbmtoppm %s");
-      else if (!strcasecmp(s, "ilb"))
-	strcpy(cmd, "%P/ilbmtoppm %s");
-      else if (!strcasecmp(s, "iff"))
-	strcpy(cmd, "%P/ilbmtoppm %s");
-      else if (!strcasecmp(s, "img"))
-	strcpy(cmd, "%P/imgtoppm %s");
-      else if (!strcasecmp(s, "mtv"))
-	strcpy(cmd, "%P/mtvtoppm %s");
-      else if (!strcasecmp(s, "pcx"))
-	strcpy(cmd, "%P/pcxtoppm %s");
-      else if (!strcasecmp(s, "pgm"))
-	strcpy(cmd, "%P/pgmtoppm rgb:ffff/ffff/ffff %s");
-      else if (!strcasecmp(s, "pi1"))
-	strcpy(cmd, "%P/pi1toppm %s");
-      else if (!strcasecmp(s, "pict"))
-	strcpy(cmd, "%P/picttoppm %s");
-      else if (!strcasecmp(s, "pic"))
-	strcpy(cmd, "%P/picttoppm %s");
-      else if (!strcasecmp(s, "pj"))
-	strcpy(cmd, "%P/pjtoppm %s");
-      else if (!strcasecmp(s, "qrt"))
-	strcpy(cmd, "%P/qrttoppm %s");
-      else if (!strcasecmp(s, "sld"))
-	strcpy(cmd, "%P/sldtoppm %s");
-      else if (!strcasecmp(s, "spc"))
-	strcpy(cmd, "%P/spctoppm %s");
-      else if (!strcasecmp(s, "spu"))
-	strcpy(cmd, "%P/sputoppm %s");
-      else if (!strcasecmp(s, "tga"))
-	strcpy(cmd, "%P/tgatoppm %s");
-      else if (!strcasecmp(s, "xim"))
-	strcpy(cmd, "%P/ximtoppm %s");
-      else if (!strcasecmp(s, "xpm"))
-	strcpy(cmd, "%P/xpmtoppm %s");
-      else if (!strcasecmp(s, "gif"))
-	strcpy(cmd, "%P/giftopnm %s");
-      else if (!strcasecmp(s, "rast"))
-	strcpy(cmd, "%P/rasttopnm %s");
-      else if (!strcasecmp(s, "ras"))
-	strcpy(cmd, "%P/rasttopnm %s");
-      else if (!strcasecmp(s, "sgi"))
-	strcpy(cmd, "%P/sgitopnm %s");
-      else if (!strcasecmp(s, "sir"))
-	strcpy(cmd, "%P/sirtopnm %s");
-      else if (!strcasecmp(s, "tiff"))
-	strcpy(cmd, "%P/tifftopnm %s");
-      else if (!strcasecmp(s, "tif"))
-	strcpy(cmd, "%P/tifftopnm %s");
-      else if (!strcasecmp(s, "wxd"))
-	strcpy(cmd, "%P/wxdtopnm %s");
-      else if (!strcasecmp(s, "zeiss"))
-	strcpy(cmd, "%P/zeisstopnm -ppm %s");
-      else if (!strcasecmp(s, "zei"))
-	strcpy(cmd, "%P/zeisstopnm -ppm %s");
-      else if (!strcasecmp(s, "zis"))
-	strcpy(cmd, "%P/zeisstopnm -ppm %s");
-      else
-	strcpy(cmd, "%P/anytopnm %s");
-      p = open_helper(cmd, fil, "rb");
-      if (p)
-	{
-	  data = _LoadPPM(id, p, &w, &h);
-	  close_helper(p);
-	}
-    }
+    if (p != stdin) 
+      fclose(p);
 
   if ((!eim) && (!data))
     {
-      fprintf(stderr, "IMLIB ERROR: Cannot load image: %s\nAll fallbacks failed.\n", fil);
+      fprintf(stderr, "IMLIB ERROR: Cannot load image: %s\n"
+      		      "All fallbacks failed.\n", fil);
       return NULL;
     }
     
@@ -1977,7 +1883,7 @@ Imlib_load_image(ImlibData * id, char *file)
 		fseek(p, size, SEEK_CUR);
 	    }
 	}
-      im->rgb_data = malloc(w * h * 3);
+      im->rgb_data = _imlib_malloc_image(w, h);
       if (!im->rgb_data)
 	{
 	  fclose(p);
