@@ -67,12 +67,13 @@ loader_png (FILE * f, int *w, int *h, int *t)
   if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
     png_set_expand(png_ptr);
   png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);
-  data = malloc(*w ** h * 3);
+  data = _gdk_malloc_image(*w, *h);
   if (!data)
     {
       png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
       return NULL;
     }
+    /* SAFE - the malloc_image will fail first */
   lines = (unsigned char **)malloc(*h * sizeof(unsigned char *));
 
   if (lines == NULL)
@@ -83,6 +84,7 @@ loader_png (FILE * f, int *w, int *h, int *t)
     }
   for (i = 0; i < *h; i++)
     {
+      /* SAFE - the malloc_image will fail first */
       if ((lines[i] = malloc(*w * (sizeof(unsigned char) * 4))) == NULL)
 	{
 	  int                 n;
@@ -304,7 +306,7 @@ inline_png(unsigned char *data, int data_size)
   if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
     png_set_expand(png_ptr);
   png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);
-  im->rgb_data = malloc(ww * hh * 3);
+  im->rgb_data = _gdk_malloc_image(ww, hh);
   if (!(im->rgb_data))
     {
       free(im->filename);
@@ -465,7 +467,7 @@ _loader_alpha_png (FILE * f, int *w, int *h, int *t, unsigned char **alpha)
 	if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
 		png_set_expand(png_ptr);
 	png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);
-	data = malloc(*w ** h * 3);
+	data = _gdk_malloc_image(*w, *h);
 	if (!data)
 	{
 		png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
@@ -651,6 +653,12 @@ saver_png (GdkImlibImage *im, char *file, GdkImlibSaveInfo *info)
 	  png_write_info(png_ptr, info_ptr);
 	  png_set_shift(png_ptr, &sig_bit);
 	  png_set_packing(png_ptr);
+	  if(im->rgb_width >= 0x40000000)
+	    {
+	      fclose(f);
+	      png_destroy_write_struct(&png_ptr, (png_infopp) NULL);
+	      return 0;
+ 	    }
 	  data = malloc(im->rgb_width * 4);
 	  if (!data)
 	    {
